# -*- coding: utf-8 -*-
##############################################################################
##	Changes:
##	1. Added "tempfield" 1-9 field and "test" field
##############################################################################

from PyQt4 import QtCore, QtGui
from aqt import mw, utils, webview
from aqt.qt import *
from anki import hooks
from aqt import deckchooser
from aqt import tagedit

import os
import tempfile

import etree.ElementTree as etree

import svgutils
from notes_from_svg import add_notes_non_overlapping, add_notes_overlapping
from config import SVG_EDIT_VERSION
# import the icons
from resources import *

image_occlusion_help_link = "http://tmbb.bitbucket.org/image-occlusion-2.html"

svg_edit_dir = os.path.join(os.path.dirname(__file__),
                            'svg-edit',
                            SVG_EDIT_VERSION)

svg_edit_path = os.path.join(svg_edit_dir,
                             'svg-editor.html')


svg_edit_url = QtCore.QUrl.fromLocalFile(svg_edit_path)
svg_edit_url_string = svg_edit_url.toString()

#Add all configuration options we know at this point:
svg_edit_url.setQueryItems([('initStroke[opacity]', '0'),
                            ('initStroke[width]', '0'),
                            ('initTool', 'rect'),
                            ('extensions', 'ext-image-occlusion.js')])

FILE_DIALOG_MESSAGE = "Choose Image"
FILE_DIALOG_FILTER = "Image Files (*.png *jpg *.jpeg *.gif)"

default_conf = {'initFill[color]': 'FFFFFF',
                'mask_fill_color': 'FF0000'}


class ImageOcc_Add(QtCore.QObject):
    def __init__(self, ed):
        super(QtCore.QObject, self).__init__()
        self.ed = ed
        self.mw = mw
        if not 'image_occlusion_conf' in mw.col.conf:
            # If addon has never been run
            self.mw.col.conf['image_occlusion_conf'] = default_conf

    def add_notes(self):
        clip = QApplication.clipboard()
        if clip.mimeData().imageData():
            handle, image_path = tempfile.mkstemp(suffix='.png')
            clip.image().save(image_path)
            self.mw.image_occlusion2_image_path = image_path
            clip.clear()
            #  Simple hack to catch an error. I still don't know
            # the cause of the error.
            try:
                self.call_ImageOcc_Editor(self.mw.image_occlusion2_image_path)
            except:
                image_path = QtGui.QFileDialog.getOpenFileName(None,  # parent
                                                      FILE_DIALOG_MESSAGE,
                                                      os.path.expanduser('~'),
                                                      FILE_DIALOG_FILTER)

        else:
            image_path = QtGui.QFileDialog.getOpenFileName(None,  # parent
                                                       FILE_DIALOG_MESSAGE,
                                                       os.path.expanduser('~'),
                                                       FILE_DIALOG_FILTER)

        # The following code is common to both branches of the 'if'
        if image_path:
            self.mw.image_occlusion2_image_path = image_path
            self.call_ImageOcc_Editor(self.mw.image_occlusion2_image_path)

    def call_ImageOcc_Editor(self, path):
        d = svgutils.image2svg(path)
        svg = d['svg']
        svg_b64 = d['svg_b64']
        height = d['height']
        width = d['width']

        try:
            mw.ImageOcc_Editor is not None
            select_rect_tool = "svgCanvas.setMode('rect'); "
            set_svg_content = 'svg_content = \'%s\'; ' % svg.replace('\n', '')
            set_canvas = 'svgCanvas.setSvgString(svg_content);'

            command = select_rect_tool + set_svg_content + set_canvas

            mw.ImageOcc_Editor.svg_edit.eval(command)
            mw.ImageOcc_Editor.show()

        except:
            initFill_color = mw.col.conf['image_occlusion_conf']['initFill[color]']
            url = svg_edit_url
            url.addQueryItem('initFill[color]', initFill_color)
            url.addQueryItem('dimensions', '{0},{1}'.format(width, height))
            url.addQueryItem('source', svg_b64)

            tags = self.ed.note.tags
            mw.ImageOcc_Editor = ImageOcc_Editor(tags)

            mw.ImageOcc_Editor.svg_edit.\
                               page().\
                               mainFrame().\
                               addToJavaScriptWindowObject("pyObj", self)
            mw.ImageOcc_Editor.svg_edit.load(url)
            mw.ImageOcc_Editor.show()

    @QtCore.pyqtSlot(str)
    def add_notes_non_overlapping(self, svg_contents):
        svg = etree.fromstring(svg_contents.encode('utf-8'))
        mask_fill_color, did, tags, header, footer, tempfield1, tempfield2, tempfield3, tempfield4, tempfield5, tempfield6, tempfield7, tempfield8, tempfield9, test = get_params_for_add_notes()
        # Add notes to the current deck of the collection:
        add_notes_non_overlapping(svg, mask_fill_color,
                                  tags, self.mw.image_occlusion2_image_path,
                                  header, footer, tempfield1, tempfield2, tempfield3, tempfield4, tempfield5, tempfield6, tempfield7, tempfield8, tempfield9, test, did)

    @QtCore.pyqtSlot(str)
    def add_notes_overlapping(self, svg_contents):
        svg = etree.fromstring(svg_contents.encode('utf-8'))
        mask_fill_color, did, tags, header, footer, tempfield1, tempfield2, tempfield3, tempfield4, tempfield5, tempfield6, tempfield7, tempfield8, tempfield9, test = get_params_for_add_notes()
        # Add notes to the current deck of the collection:
        add_notes_overlapping(svg, mask_fill_color,
                              tags, self.mw.image_occlusion2_image_path,
                              header, footer, tempfield1, tempfield2, tempfield3, tempfield4, tempfield5, tempfield6, tempfield7, tempfield8, tempfield9, test, did)


def get_params_for_add_notes():
    # Get the mask color from mw.col.conf:
    mask_fill_color = mw.col.conf['image_occlusion_conf']['mask_fill_color']

    header = mw.ImageOcc_Editor.header_edit.text()
    footer = mw.ImageOcc_Editor.footer_edit.text()
    tempfield1 = mw.ImageOcc_Editor.tempfield1_edit.text()
    tempfield2 = mw.ImageOcc_Editor.tempfield2_edit.text()
    tempfield3 = mw.ImageOcc_Editor.tempfield3_edit.text()
    tempfield4 = mw.ImageOcc_Editor.tempfield4_edit.text()
    tempfield5 = mw.ImageOcc_Editor.tempfield5_edit.text()
    tempfield6 = mw.ImageOcc_Editor.tempfield6_edit.text()
    tempfield7 = mw.ImageOcc_Editor.tempfield7_edit.text()
    tempfield8 = mw.ImageOcc_Editor.tempfield8_edit.text()
    tempfield9 = mw.ImageOcc_Editor.tempfield9_edit.text()
    test = mw.ImageOcc_Editor.test_edit.text()
	
    # Get deck id:
    did = mw.ImageOcc_Editor.deckChooser.selectedId()
    tags = mw.ImageOcc_Editor.tags_edit.text().split()
    return (mask_fill_color, did, tags, header, footer, tempfield1, tempfield2, tempfield3, tempfield4, tempfield5, tempfield6, tempfield7, tempfield8, tempfield9, test)


def add_image_occlusion_button(ed):
    ed.image_occlusion = ImageOcc_Add(ed)
    ed._addButton("new_occlusion", ed.image_occlusion.add_notes,
            key="Alt+o", size=False,  # text=_("Image Occlusion"),
            native=True, canDisable=False)


class ImageOcc_Editor(QWidget):
    def __init__(self, tags):
        super(ImageOcc_Editor, self).__init__()
        self.initUI(tags)

    def initUI(self, tags):
        ##############################################
        ### From top to bottom:
        ##############################################
        ## header_label # self.header_edit
        ##############################################
        ## self.svg_edit
        ##############################################
        ## footer_label # self.footer
        ##############################################
        ## tempfield1_label # self.tempfield1
        ##############################################
        ## tempfield2_label # self.tempfield2
        ##############################################
        ## tempfield3_label # self.tempfield3
        ##############################################
        ## tempfield4_label # self.tempfield4
        ##############################################
        ## tempfield5_label # self.tempfield5
        ##############################################
        ## tempfield6_label # self.tempfield6
        ##############################################
        ## tempfield7_label # self.tempfield7
        ##############################################
        ## tempfield8_label # self.tempfield8
        ##############################################
        ## tempfield9_label # self.tempfield9
        ##############################################
        ## test_label # self.test
        ##############################################
        ## tags_label # self.tags_edit
        ##############################################
        ## deck_container
        ##  ########################
        ##  # deckChooser          #
        ##  ########################
        ##############################################

        # Header
        self.header_edit = QLineEdit()
        self.header_edit.setPlaceholderText("(optional)")
        header_label = QLabel("Header: ")

        header_hbox = QHBoxLayout()
        header_hbox.addWidget(header_label)
        header_hbox.addWidget(self.header_edit)

        # svg-edit
        self.svg_edit = webview.AnkiWebView()
        self.svg_edit.setCanFocus(True)

        # Footer
        self.footer_edit = QLineEdit()
        self.footer_edit.setPlaceholderText("(optional)")
        footer_label = QLabel("Footer: ")

        footer_hbox = QHBoxLayout()
        footer_hbox.addWidget(footer_label)
        footer_hbox.addWidget(self.footer_edit)

        # TempField1
        self.tempfield1_edit = QLineEdit()
        self.tempfield1_edit.setPlaceholderText("(optional)")
        tempfield1_label = QLabel("TempField1: ")

        tempfield1_hbox = QHBoxLayout()
        tempfield1_hbox.addWidget(tempfield1_label)
        tempfield1_hbox.addWidget(self.tempfield1_edit)

        # TempField2
        self.tempfield2_edit = QLineEdit()
        self.tempfield2_edit.setPlaceholderText("(optional)")
        tempfield2_label = QLabel("TempField2: ")

        tempfield2_hbox = QHBoxLayout()
        tempfield2_hbox.addWidget(tempfield2_label)
        tempfield2_hbox.addWidget(self.tempfield2_edit)

        # TempField3
        self.tempfield3_edit = QLineEdit()
        self.tempfield3_edit.setPlaceholderText("(optional)")
        tempfield3_label = QLabel("TempField3: ")

        tempfield3_hbox = QHBoxLayout()
        tempfield3_hbox.addWidget(tempfield3_label)
        tempfield3_hbox.addWidget(self.tempfield3_edit)

        # TempField4
        self.tempfield4_edit = QLineEdit()
        self.tempfield4_edit.setPlaceholderText("(optional)")
        tempfield4_label = QLabel("TempField4: ")

        tempfield4_hbox = QHBoxLayout()
        tempfield4_hbox.addWidget(tempfield4_label)
        tempfield4_hbox.addWidget(self.tempfield4_edit)

        # TempField5
        self.tempfield5_edit = QLineEdit()
        self.tempfield5_edit.setPlaceholderText("(optional)")
        tempfield5_label = QLabel("TempField5: ")

        tempfield5_hbox = QHBoxLayout()
        tempfield5_hbox.addWidget(tempfield5_label)
        tempfield5_hbox.addWidget(self.tempfield5_edit)

        # TempField6
        self.tempfield6_edit = QLineEdit()
        self.tempfield6_edit.setPlaceholderText("(optional)")
        tempfield6_label = QLabel("TempField6: ")

        tempfield6_hbox = QHBoxLayout()
        tempfield6_hbox.addWidget(tempfield6_label)
        tempfield6_hbox.addWidget(self.tempfield6_edit)

        # TempField7
        self.tempfield7_edit = QLineEdit()
        self.tempfield7_edit.setPlaceholderText("(optional)")
        tempfield7_label = QLabel("TempField7: ")

        tempfield7_hbox = QHBoxLayout()
        tempfield7_hbox.addWidget(tempfield7_label)
        tempfield7_hbox.addWidget(self.tempfield7_edit)

        # TempField8
        self.tempfield8_edit = QLineEdit()
        self.tempfield8_edit.setPlaceholderText("(optional)")
        tempfield8_label = QLabel("TempField8: ")

        tempfield8_hbox = QHBoxLayout()
        tempfield8_hbox.addWidget(tempfield8_label)
        tempfield8_hbox.addWidget(self.tempfield8_edit)

        # TempField9
        self.tempfield9_edit = QLineEdit()
        self.tempfield9_edit.setPlaceholderText("(optional)")
        tempfield9_label = QLabel("TempField9: ")

        tempfield9_hbox = QHBoxLayout()
        tempfield9_hbox.addWidget(tempfield9_label)
        tempfield9_hbox.addWidget(self.tempfield9_edit)

        # Test
        self.test_edit = QLineEdit()
        self.test_edit.setPlaceholderText("(optional)")
        test_label = QLabel("Test: ")

        test_hbox = QHBoxLayout()
        test_hbox.addWidget(test_label)		
        test_hbox.addWidget(self.test_edit)

        # Tags
        self.tags_edit = tagedit.TagEdit(self)
        self.tags_edit.setText(" ".join(tags))
        self.tags_edit.setCol(mw.col)
        tags_label = QLabel("Tags: ")

        tags_hbox = QHBoxLayout()
        tags_hbox.addWidget(tags_label)
        tags_hbox.addWidget(self.tags_edit)

        deck_container = QGroupBox("Deck")

        self.deckChooser = deckchooser.DeckChooser(mw, deck_container,
                                                   label=False)

        vbox = QVBoxLayout()
        vbox.addLayout(header_hbox)
        vbox.addWidget(self.svg_edit, stretch=1)
        vbox.addLayout(footer_hbox)
        vbox.addLayout(tempfield1_hbox)
        vbox.addLayout(tempfield2_hbox)
        vbox.addLayout(tempfield3_hbox)
        vbox.addLayout(tempfield4_hbox)
        vbox.addLayout(tempfield5_hbox)
        vbox.addLayout(tempfield6_hbox)
        vbox.addLayout(tempfield7_hbox)
        vbox.addLayout(tempfield8_hbox)
        vbox.addLayout(tempfield9_hbox)
        vbox.addLayout(test_hbox)
        vbox.addLayout(tags_hbox)
        vbox.addWidget(deck_container)

        self.setLayout(vbox)
        self.setMinimumHeight(600)
        #  Don't focus on the tags.
        #  When the focus is on the tags an ugly autocomplete
        # list appears, taking away screen real estate for no reason.
        self.header_edit.setFocus()

        self.setWindowTitle('Image Occlusion Editor')
        self.show()


class ImageOcc_Options(QtGui.QWidget):
    def __init__(self, mw):
        super(ImageOcc_Options, self).__init__()
        self.mw = mw

    def getNewMaskColor(self):
        # Remove the # sign from QColor.name():
        choose_color_dialog = QColorDialog()
        color = choose_color_dialog.getColor()
        if color.isValid():
            color_ = color.name()[1:]
            self.mw.col.conf['image_occlusion_conf']['mask_fill_color'] = color_

    def getNewInitFillColor(self):
        # Remove the # sign from QColor.name():
        choose_color_dialog = QColorDialog()
        color = choose_color_dialog.getColor()
        if color.isValid():
            color_ = color.name()[1:]
            self.mw.col.conf['image_occlusion_conf']['initFill[color]'] = color_

    def setupUi(self):

        ### Mask color for questions:
        mask_color_label = QLabel('<b>Mask color</b><br>in question')

        mask_color_button = QPushButton(u"Choose Color ▾")

        mask_color_button.connect(mask_color_button,
                                  SIGNAL("clicked()"),
                                  self.getNewMaskColor)
        ### Initial rectangle color:
        initFill_label = QLabel('<b>Initial color</b><br>for rectangle')

        initFill_button = QPushButton(u"Choose Color ▾")

        initFill_button.connect(initFill_button,
                                SIGNAL("clicked()"),
                                self.getNewInitFillColor)

        grid = QtGui.QGridLayout()
        grid.setSpacing(10)

        # 1st row:
        grid.addWidget(mask_color_label, 0, 0)
        grid.addWidget(mask_color_button, 0, 1)
        # 2nd row:
        grid.addWidget(initFill_label, 1, 0)
        grid.addWidget(initFill_button, 1, 1)

        self.setLayout(grid)

        self.setWindowTitle('Image Occlusion 2.0 (options)')
        self.show()


def invoke_ImageOcc_help():
    utils.openLink(image_occlusion_help_link)

mw.ImageOcc_Options = ImageOcc_Options(mw)

options_action = QAction("Image Occlusion 2.0 (options)", mw)
help_action = QAction("Image Occlusion 2.0 (help)", mw)

mw.connect(options_action,
           SIGNAL("triggered()"),
           mw.ImageOcc_Options.setupUi)

mw.connect(help_action,
           SIGNAL("triggered()"),
           invoke_ImageOcc_help)

mw.form.menuTools.addAction(options_action)
mw.form.menuHelp.addAction(help_action)

hooks.addHook('setupEditorButtons', add_image_occlusion_button)
